@@ -122,9 +122,11 @@ const (
 	KeyLogDriver             = "LogDriver"
 	KeyLogOpt                = "LogOpt"
 	KeyMask                  = "Mask"
+	KeyMemory                = "Memory"
 	KeyMount                 = "Mount"
 	KeyNetwork               = "Network"
 	KeyNetworkAlias          = "NetworkAlias"
+	KeyNetworkDeleteOnStop   = "NetworkDeleteOnStop"
 	KeyNetworkName           = "NetworkName"
 	KeyNoNewPrivileges       = "NoNewPrivileges"
 	KeyNotify                = "Notify"
@@ -138,10 +140,14 @@ const (
 	KeyPull                  = "Pull"
 	KeyReadOnly              = "ReadOnly"
 	KeyReadOnlyTmpfs         = "ReadOnlyTmpfs"
-	KeyRemapGid              = "RemapGid"     //nolint:stylecheck // deprecated
-	KeyRemapUid              = "RemapUid"     //nolint:stylecheck // deprecated
-	KeyRemapUidSize          = "RemapUidSize" //nolint:stylecheck // deprecated
+	KeyReloadCmd             = "ReloadCmd"
+	KeyReloadSignal          = "ReloadSignal"
+	KeyRemapGid              = "RemapGid"     // deprecated
+	KeyRemapUid              = "RemapUid"     // deprecated
+	KeyRemapUidSize          = "RemapUidSize" // deprecated
 	KeyRemapUsers            = "RemapUsers"   // deprecated
+	KeyRetry                 = "Retry"
+	KeyRetryDelay            = "RetryDelay"
 	KeyRootfs                = "Rootfs"
 	KeyRunInit               = "RunInit"
 	KeySeccompProfile        = "SeccompProfile"
@@ -191,9 +197,39 @@ type UnitInfo struct {
 }
 
 var (
+	// Key: Extension
+	// Value: Processing order for resource naming dependencies
+	SupportedExtensions = map[string]int{
+		".container": 4,
+		".volume":    2,
+		".kube":      4,
+		".network":   2,
+		".image":     1,
+		".build":     3,
+		".pod":       5,
+	}
+
 	URL            = regexp.Delayed(`^((https?)|(git)://)|(github\.com/).+$`)
 	validPortRange = regexp.Delayed(`\d+(-\d+)?(/udp|/tcp)?$`)
 
+	unitDependencyKeys = []string{
+		"After",
+		"Before",
+		"BindsTo",
+		"Conflicts",
+		"OnFailure",
+		"OnSuccess",
+		"PartOf",
+		"PropagatesReloadTo",
+		"PropagatesStopTo",
+		"ReloadPropagatedFrom",
+		"Requires",
+		"Requisite",
+		"StopPropagatedFrom",
+		"Upholds",
+		"Wants",
+	}
+
 	// Supported keys in "Container" group
 	supportedContainerKeys = map[string]bool{
 		KeyAddCapability:         true,
@@ -240,6 +276,7 @@ var (
 		KeyLogDriver:             true,
 		KeyLogOpt:                true,
 		KeyMask:                  true,
+		KeyMemory:                true,
 		KeyMount:                 true,
 		KeyNetwork:               true,
 		KeyNetworkAlias:          true,
@@ -252,10 +289,14 @@ var (
 		KeyPull:                  true,
 		KeyReadOnly:              true,
 		KeyReadOnlyTmpfs:         true,
+		KeyReloadCmd:             true,
+		KeyReloadSignal:          true,
 		KeyRemapGid:              true,
 		KeyRemapUid:              true,
 		KeyRemapUidSize:          true,
 		KeyRemapUsers:            true,
+		KeyRetry:                 true,
+		KeyRetryDelay:            true,
 		KeyRootfs:                true,
 		KeyRunInit:               true,
 		KeySeccompProfile:        true,
@@ -317,6 +358,7 @@ var (
 		KeyIPv6:                 true,
 		KeyInternal:             true,
 		KeyNetworkName:          true,
+		KeyNetworkDeleteOnStop:  true,
 		KeyOptions:              true,
 		KeyServiceName:          true,
 		KeySubnet:               true,
@@ -360,6 +402,8 @@ var (
 		KeyImageTag:             true,
 		KeyOS:                   true,
 		KeyPodmanArgs:           true,
+		KeyRetry:                true,
+		KeyRetryDelay:           true,
 		KeyServiceName:          true,
 		KeyTLSVerify:            true,
 		KeyVariant:              true,
@@ -384,6 +428,8 @@ var (
 		KeyNetwork:              true,
 		KeyPodmanArgs:           true,
 		KeyPull:                 true,
+		KeyRetry:                true,
+		KeyRetryDelay:           true,
 		KeySecret:               true,
 		KeyServiceName:          true,
 		KeySetWorkingDirectory:  true,
@@ -401,6 +447,7 @@ var (
 		KeyDNSSearch:            true,
 		KeyGIDMap:               true,
 		KeyGlobalArgs:           true,
+		KeyHostName:             true,
 		KeyIP:                   true,
 		KeyIP6:                  true,
 		KeyNetwork:              true,
@@ -413,6 +460,7 @@ var (
 		KeyRemapUidSize:         true,
 		KeyRemapUsers:           true,
 		KeyServiceName:          true,
+		KeyShmSize:              true,
 		KeySubGIDMap:            true,
 		KeySubUIDMap:            true,
 		KeyUIDMap:               true,
@@ -469,35 +517,6 @@ func checkForUnknownKeys(unit *parser.UnitFile, groupName string, supportedKeys
 	return err
 }
 
-func splitPorts(ports string) []string {
-	parts := make([]string, 0)
-
-	// IP address could have colons in it. For example: "[::]:8080:80/tcp, so we split carefully
-	start := 0
-	end := 0
-	for end < len(ports) {
-		switch ports[end] {
-		case '[':
-			end++
-			for end < len(ports) && ports[end] != ']' {
-				end++
-			}
-			if end < len(ports) {
-				end++ // Skip ]
-			}
-		case ':':
-			parts = append(parts, ports[start:end])
-			end++
-			start = end
-		default:
-			end++
-		}
-	}
-
-	parts = append(parts, ports[start:end])
-	return parts
-}
-
 func usernsOpts(kind string, opts []string) string {
 	var res strings.Builder
 	res.WriteString(kind)
@@ -517,15 +536,21 @@ func usernsOpts(kind string, opts []string) string {
 // service file (unit file with Service group) based on the options in the
 // Container group.
 // The original Container group is kept around as X-Container.
-func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[string]*UnitInfo) (*parser.UnitFile, error) {
+func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[string]*UnitInfo) (*parser.UnitFile, error, error) {
+	var warn, warnings error
+
 	unitInfo, ok := unitsInfoMap[container.Filename]
 	if !ok {
-		return nil, fmt.Errorf("internal error while processing container %s", container.Filename)
+		return nil, warnings, fmt.Errorf("internal error while processing container %s", container.Filename)
 	}
 
 	service := container.Dup()
 	service.Filename = unitInfo.ServiceFileName()
 
+	if err := translateUnitDependencies(service, unitsInfoMap); err != nil {
+		return nil, warnings, err
+	}
+
 	addDefaultDependencies(service, isUser)
 
 	if container.Path != "" {
@@ -533,7 +558,7 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 	}
 
 	if err := checkForUnknownKeys(container, ContainerGroup, supportedContainerKeys); err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	// Rename old Container group to x-Container so that systemd ignores it
@@ -546,37 +571,29 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 	image, _ := container.Lookup(ContainerGroup, KeyImage)
 	rootfs, _ := container.Lookup(ContainerGroup, KeyRootfs)
 	if len(image) == 0 && len(rootfs) == 0 {
-		return nil, fmt.Errorf("no Image or Rootfs key specified")
+		return nil, warnings, fmt.Errorf("no Image or Rootfs key specified")
 	}
 	if len(image) > 0 && len(rootfs) > 0 {
-		return nil, fmt.Errorf("the Image And Rootfs keys conflict can not be specified together")
+		return nil, warnings, fmt.Errorf("the Image And Rootfs keys conflict can not be specified together")
 	}
 
 	if len(image) > 0 {
 		var err error
 		if image, err = handleImageSource(image, service, unitsInfoMap); err != nil {
-			return nil, err
+			return nil, warnings, err
 		}
 	}
 
-	containerName, ok := container.Lookup(ContainerGroup, KeyContainerName)
-	if !ok || len(containerName) == 0 {
-		// By default, We want to name the container by the service name
-		if strings.Contains(container.Filename, "@") {
-			containerName = "systemd-%p_%i"
-		} else {
-			containerName = "systemd-%N"
-		}
-	}
+	containerName := getContainerName(container)
 
 	// Set PODMAN_SYSTEMD_UNIT so that podman auto-update can restart the service.
 	service.Add(ServiceGroup, "Environment", "PODMAN_SYSTEMD_UNIT=%n")
 
 	// Only allow mixed or control-group, as nothing else works well
 	killMode, ok := service.Lookup(ServiceGroup, "KillMode")
-	if !ok || !(killMode == "mixed" || killMode == "control-group") {
+	if !ok || (killMode != "mixed" && killMode != "control-group") {
 		if ok {
-			return nil, fmt.Errorf("invalid KillMode '%s'", killMode)
+			return nil, warnings, fmt.Errorf("invalid KillMode '%s'", killMode)
 		}
 
 		// We default to mixed instead of control-group, because it lets conmon do its thing
@@ -584,7 +601,8 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 	}
 
 	// Read env early so we can override it below
-	podmanEnv := container.LookupAllKeyVal(ContainerGroup, KeyEnvironment)
+	podmanEnv, warn := container.LookupAllKeyVal(ContainerGroup, KeyEnvironment)
+	warnings = errors.Join(warnings, warn)
 
 	// Need the containers filesystem mounted to start podman
 	service.Add(UnitGroup, "RequiresMountsFor", "%t/containers")
@@ -601,6 +619,10 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 	serviceStopCmd.Args[0] = fmt.Sprintf("-%s", serviceStopCmd.Args[0])
 	service.AddCmdline(ServiceGroup, "ExecStopPost", serviceStopCmd.Args)
 
+	if err := handleExecReload(container, service, ContainerGroup); err != nil {
+		return nil, warnings, err
+	}
+
 	podman := createBasePodmanCommand(container, ContainerGroup)
 
 	podman.add("run")
@@ -642,6 +664,9 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 		KeyStopSignal:  "--stop-signal",
 		KeyStopTimeout: "--stop-timeout",
 		KeyPull:        "--pull",
+		KeyMemory:      "--memory",
+		KeyRetry:       "--retry",
+		KeyRetryDelay:  "--retry-delay",
 	}
 	lookupAndAddString(container, ContainerGroup, stringKeys, podman)
 
@@ -665,12 +690,12 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 	lookupAndAddBoolean(container, ContainerGroup, boolKeys, podman)
 
 	if err := addNetworks(container, ContainerGroup, service, unitsInfoMap, podman); err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	serviceType, ok := service.Lookup(ServiceGroup, "Type")
 	if ok && serviceType != "notify" && serviceType != "oneshot" {
-		return nil, fmt.Errorf("invalid service Type '%s'", serviceType)
+		return nil, warnings, fmt.Errorf("invalid service Type '%s'", serviceType)
 	}
 
 	if serviceType != "oneshot" {
@@ -775,15 +800,15 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 	}
 
 	if err := handleUser(container, ContainerGroup, podman); err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	if err := handleUserMappings(container, ContainerGroup, podman, true); err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	if err := addVolumes(container, service, ContainerGroup, unitsInfoMap, podman); err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	update, ok := container.Lookup(ContainerGroup, KeyAutoUpdate)
@@ -798,7 +823,7 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 		exposedPort = strings.TrimSpace(exposedPort) // Allow whitespace after
 
 		if !isPortRange(exposedPort) {
-			return nil, fmt.Errorf("invalid port format '%s'", exposedPort)
+			return nil, warnings, fmt.Errorf("invalid port format '%s'", exposedPort)
 		}
 
 		podman.add("--expose", exposedPort)
@@ -808,10 +833,12 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 
 	podman.addEnv(podmanEnv)
 
-	labels := container.LookupAllKeyVal(ContainerGroup, KeyLabel)
+	labels, warn := container.LookupAllKeyVal(ContainerGroup, KeyLabel)
+	warnings = errors.Join(warnings, warn)
 	podman.addLabels(labels)
 
-	annotations := container.LookupAllKeyVal(ContainerGroup, KeyAnnotation)
+	annotations, warn := container.LookupAllKeyVal(ContainerGroup, KeyAnnotation)
+	warnings = errors.Join(warnings, warn)
 	podman.addAnnotations(annotations)
 
 	masks := container.LookupAllArgs(ContainerGroup, KeyMask)
@@ -828,7 +855,7 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 	for _, envFile := range envFiles {
 		filePath, err := getAbsolutePath(container, envFile)
 		if err != nil {
-			return nil, err
+			return nil, warnings, err
 		}
 		podman.add("--env-file", filePath)
 	}
@@ -842,7 +869,7 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 	for _, mount := range mounts {
 		mountStr, err := resolveContainerMountParams(container, service, mount, unitsInfoMap)
 		if err != nil {
-			return nil, err
+			return nil, warnings, err
 		}
 		podman.add("--mount", mountStr)
 	}
@@ -850,7 +877,7 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 	handleHealth(container, ContainerGroup, podman)
 
 	if err := handlePod(container, service, ContainerGroup, unitsInfoMap, podman); err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	handlePodmanArgs(container, ContainerGroup, podman)
@@ -868,14 +895,37 @@ func ConvertContainer(container *parser.UnitFile, isUser bool, unitsInfoMap map[
 
 	service.AddCmdline(ServiceGroup, "ExecStart", podman.Args)
 
+	return service, warnings, nil
+}
+
+// Get the unresolved container name that may contain '%'.
+func getContainerName(container *parser.UnitFile) string {
+	containerName, ok := container.Lookup(ContainerGroup, KeyContainerName)
+	if !ok || len(containerName) == 0 {
+		// By default, We want to name the container by the service name.
+		if strings.Contains(container.Filename, "@") {
+			containerName = "systemd-%p_%i"
+		} else {
+			containerName = "systemd-%N"
+		}
+	}
+	return containerName
+}
+
+// Get the resolved container name that contains no '%'.
+// Returns an empty string if not resolvable.
+func GetContainerResourceName(container *parser.UnitFile) string {
+	containerName := getContainerName(container)
+
 	// XXX: only %N is handled.
 	// it is difficult to properly implement specifiers handling without consulting systemd.
-	resourceName := strings.ReplaceAll(containerName, "%N", unitInfo.ServiceName)
-	if !strings.Contains(resourceName, "%") {
-		unitInfo.ResourceName = resourceName
-	}
+	resourceName := strings.ReplaceAll(containerName, "%N", GetContainerServiceName(container))
 
-	return service, nil
+	if !strings.Contains(resourceName, "%") {
+		return resourceName
+	} else {
+		return ""
+	}
 }
 
 func defaultOneshotServiceGroup(service *parser.UnitFile, remainAfterExit bool) {
@@ -899,15 +949,21 @@ func defaultOneshotServiceGroup(service *parser.UnitFile, remainAfterExit bool)
 // The original Network group is kept around as X-Network.
 // Also returns the canonical network name, either auto-generated or user-defined via the
 // NetworkName key-value.
-func ConvertNetwork(network *parser.UnitFile, name string, unitsInfoMap map[string]*UnitInfo, isUser bool) (*parser.UnitFile, error) {
+func ConvertNetwork(network *parser.UnitFile, name string, unitsInfoMap map[string]*UnitInfo, isUser bool) (*parser.UnitFile, error, error) {
+	var warn, warnings error
+
 	unitInfo, ok := unitsInfoMap[network.Filename]
 	if !ok {
-		return nil, fmt.Errorf("internal error while processing network %s", network.Filename)
+		return nil, warnings, fmt.Errorf("internal error while processing network %s", network.Filename)
 	}
 
 	service := network.Dup()
 	service.Filename = unitInfo.ServiceFileName()
 
+	if err := translateUnitDependencies(service, unitsInfoMap); err != nil {
+		return nil, warnings, err
+	}
+
 	addDefaultDependencies(service, isUser)
 
 	if network.Path != "" {
@@ -915,7 +971,7 @@ func ConvertNetwork(network *parser.UnitFile, name string, unitsInfoMap map[stri
 	}
 
 	if err := checkForUnknownKeys(network, NetworkGroup, supportedNetworkKeys); err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	/* Rename old Network group to x-Network so that systemd ignores it */
@@ -933,6 +989,12 @@ func ConvertNetwork(network *parser.UnitFile, name string, unitsInfoMap map[stri
 	// Need the containers filesystem mounted to start podman
 	service.Add(UnitGroup, "RequiresMountsFor", "%t/containers")
 
+	if network.LookupBooleanWithDefault(NetworkGroup, KeyNetworkDeleteOnStop, false) {
+		serviceStopPostCmd := createBasePodmanCommand(network, NetworkGroup)
+		serviceStopPostCmd.add("network", "rm", networkName)
+		service.AddCmdline(ServiceGroup, "ExecStopPost", serviceStopPostCmd.Args)
+	}
+
 	podman := createBasePodmanCommand(network, NetworkGroup)
 
 	podman.add("network", "create", "--ignore")
@@ -960,10 +1022,10 @@ func ConvertNetwork(network *parser.UnitFile, name string, unitsInfoMap map[stri
 	ipRanges := network.LookupAll(NetworkGroup, KeyIPRange)
 	if len(subnets) > 0 {
 		if len(gateways) > len(subnets) {
-			return nil, fmt.Errorf("cannot set more gateways than subnets")
+			return nil, warnings, fmt.Errorf("cannot set more gateways than subnets")
 		}
 		if len(ipRanges) > len(subnets) {
-			return nil, fmt.Errorf("cannot set more ranges than subnets")
+			return nil, warnings, fmt.Errorf("cannot set more ranges than subnets")
 		}
 		for i := range subnets {
 			podman.add("--subnet", subnets[i])
@@ -975,15 +1037,18 @@ func ConvertNetwork(network *parser.UnitFile, name string, unitsInfoMap map[stri
 			}
 		}
 	} else if len(ipRanges) > 0 || len(gateways) > 0 {
-		return nil, fmt.Errorf("cannot set gateway or range without subnet")
+		return nil, warnings, fmt.Errorf("cannot set gateway or range without subnet")
 	}
 
-	networkOptions := network.LookupAllKeyVal(NetworkGroup, KeyOptions)
+	networkOptions, warn := network.LookupAllKeyVal(NetworkGroup, KeyOptions)
+	warnings = errors.Join(warnings, warn)
 	if len(networkOptions) > 0 {
 		podman.addKeys("--opt", networkOptions)
 	}
 
-	if labels := network.LookupAllKeyVal(NetworkGroup, KeyLabel); len(labels) > 0 {
+	labels, warn := network.LookupAllKeyVal(NetworkGroup, KeyLabel)
+	warnings = errors.Join(warnings, warn)
+	if len(labels) > 0 {
 		podman.addLabels(labels)
 	}
 
@@ -997,7 +1062,7 @@ func ConvertNetwork(network *parser.UnitFile, name string, unitsInfoMap map[stri
 
 	// Store the name of the created resource
 	unitInfo.ResourceName = networkName
-	return service, nil
+	return service, warnings, nil
 }
 
 // Convert a quadlet volume file (unit file with a Volume group) to a systemd
@@ -1006,15 +1071,20 @@ func ConvertNetwork(network *parser.UnitFile, name string, unitsInfoMap map[stri
 // The original Volume group is kept around as X-Volume.
 // Also returns the canonical volume name, either auto-generated or user-defined via the VolumeName
 // key-value.
-func ConvertVolume(volume *parser.UnitFile, name string, unitsInfoMap map[string]*UnitInfo, isUser bool) (*parser.UnitFile, error) {
+func ConvertVolume(volume *parser.UnitFile, name string, unitsInfoMap map[string]*UnitInfo, isUser bool) (*parser.UnitFile, error, error) {
+	var warn, warnings error
 	unitInfo, ok := unitsInfoMap[volume.Filename]
 	if !ok {
-		return nil, fmt.Errorf("internal error while processing network %s", volume.Filename)
+		return nil, warnings, fmt.Errorf("internal error while processing network %s", volume.Filename)
 	}
 
 	service := volume.Dup()
 	service.Filename = unitInfo.ServiceFileName()
 
+	if err := translateUnitDependencies(service, unitsInfoMap); err != nil {
+		return nil, warnings, err
+	}
+
 	addDefaultDependencies(service, isUser)
 
 	if volume.Path != "" {
@@ -1022,7 +1092,7 @@ func ConvertVolume(volume *parser.UnitFile, name string, unitsInfoMap map[string
 	}
 
 	if err := checkForUnknownKeys(volume, VolumeGroup, supportedVolumeKeys); err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	/* Rename old Volume group to x-Volume so that systemd ignores it */
@@ -1040,7 +1110,8 @@ func ConvertVolume(volume *parser.UnitFile, name string, unitsInfoMap map[string
 	// Need the containers filesystem mounted to start podman
 	service.Add(UnitGroup, "RequiresMountsFor", "%t/containers")
 
-	labels := volume.LookupAllKeyVal(VolumeGroup, "Label")
+	labels, warn := volume.LookupAllKeyVal(VolumeGroup, "Label")
+	warnings = errors.Join(warnings, warn)
 
 	podman := createBasePodmanCommand(volume, VolumeGroup)
 
@@ -1058,11 +1129,11 @@ func ConvertVolume(volume *parser.UnitFile, name string, unitsInfoMap map[string
 
 		imageName, ok := volume.Lookup(VolumeGroup, KeyImage)
 		if !ok {
-			return nil, fmt.Errorf("the key %s is mandatory when using the image driver", KeyImage)
+			return nil, warnings, fmt.Errorf("the key %s is mandatory when using the image driver", KeyImage)
 		}
 		imageName, err := handleImageSource(imageName, service, unitsInfoMap)
 		if err != nil {
-			return nil, err
+			return nil, warnings, err
 		}
 
 		opts.WriteString(imageName)
@@ -1107,7 +1178,7 @@ func ConvertVolume(volume *parser.UnitFile, name string, unitsInfoMap map[string
 			if devValid {
 				podman.add("--opt", fmt.Sprintf("type=%s", devType))
 			} else {
-				return nil, fmt.Errorf("key Type can't be used without Device")
+				return nil, warnings, fmt.Errorf("key Type can't be used without Device")
 			}
 		}
 
@@ -1119,7 +1190,7 @@ func ConvertVolume(volume *parser.UnitFile, name string, unitsInfoMap map[string
 				}
 				opts.WriteString(mountOpts)
 			} else {
-				return nil, fmt.Errorf("key Options can't be used without Device")
+				return nil, warnings, fmt.Errorf("key Options can't be used without Device")
 			}
 		}
 	}
@@ -1141,7 +1212,7 @@ func ConvertVolume(volume *parser.UnitFile, name string, unitsInfoMap map[string
 	// Store the name of the created resource
 	unitInfo.ResourceName = volumeName
 
-	return service, nil
+	return service, warnings, nil
 }
 
 func ConvertKube(kube *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isUser bool) (*parser.UnitFile, error) {
@@ -1153,6 +1224,10 @@ func ConvertKube(kube *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isUse
 	service := kube.Dup()
 	service.Filename = unitInfo.ServiceFileName()
 
+	if err := translateUnitDependencies(service, unitsInfoMap); err != nil {
+		return nil, err
+	}
+
 	addDefaultDependencies(service, isUser)
 
 	if kube.Path != "" {
@@ -1181,7 +1256,7 @@ func ConvertKube(kube *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isUse
 
 	// Only allow mixed or control-group, as nothing else works well
 	killMode, ok := service.Lookup(ServiceGroup, "KillMode")
-	if !ok || !(killMode == "mixed" || killMode == "control-group") {
+	if !ok || (killMode != "mixed" && killMode != "control-group") {
 		if ok {
 			return nil, fmt.Errorf("invalid KillMode '%s'", killMode)
 		}
@@ -1298,6 +1373,10 @@ func ConvertImage(image *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isU
 	service := image.Dup()
 	service.Filename = unitInfo.ServiceFileName()
 
+	if err := translateUnitDependencies(service, unitsInfoMap); err != nil {
+		return nil, err
+	}
+
 	addDefaultDependencies(service, isUser)
 
 	if image.Path != "" {
@@ -1334,6 +1413,8 @@ func ConvertImage(image *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isU
 		KeyDecryptionKey: "--decryption-key",
 		KeyOS:            "--os",
 		KeyVariant:       "--variant",
+		KeyRetry:         "--retry",
+		KeyRetryDelay:    "--retry-delay",
 	}
 	lookupAndAddString(image, ImageGroup, stringKeys, podman)
 
@@ -1361,20 +1442,26 @@ func ConvertImage(image *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isU
 	return service, nil
 }
 
-func ConvertBuild(build *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isUser bool) (*parser.UnitFile, error) {
+func ConvertBuild(build *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isUser bool) (*parser.UnitFile, error, error) {
+	var warn, warnings error
+
 	unitInfo, ok := unitsInfoMap[build.Filename]
 	if !ok {
-		return nil, fmt.Errorf("internal error while processing network %s", build.Filename)
+		return nil, warnings, fmt.Errorf("internal error while processing network %s", build.Filename)
 	}
 
 	// Fast fail is ResouceName is not set
 	if len(unitInfo.ResourceName) == 0 {
-		return nil, fmt.Errorf("no ImageTag key specified")
+		return nil, warnings, fmt.Errorf("no ImageTag key specified")
 	}
 
 	service := build.Dup()
 	service.Filename = unitInfo.ServiceFileName()
 
+	if err := translateUnitDependencies(service, unitsInfoMap); err != nil {
+		return nil, warnings, err
+	}
+
 	addDefaultDependencies(service, isUser)
 
 	/* Rename old Build group to X-Build so that systemd ignores it */
@@ -1391,18 +1478,25 @@ func ConvertBuild(build *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isU
 	}
 
 	if err := checkForUnknownKeys(build, BuildGroup, supportedBuildKeys); err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	podman := createBasePodmanCommand(build, BuildGroup)
 	podman.add("build")
 
+	// The `--pull` flag has to be handled separately and the `=` sign must be present
+	// See https://github.com/containers/podman/issues/24599 for details
+	if val, ok := build.Lookup(BuildGroup, KeyPull); ok && len(val) > 0 {
+		podman.addf("--pull=%s", val)
+	}
+
 	stringKeys := map[string]string{
-		KeyArch:     "--arch",
-		KeyAuthFile: "--authfile",
-		KeyPull:     "--pull",
-		KeyTarget:   "--target",
-		KeyVariant:  "--variant",
+		KeyArch:       "--arch",
+		KeyAuthFile:   "--authfile",
+		KeyTarget:     "--target",
+		KeyVariant:    "--variant",
+		KeyRetry:      "--retry",
+		KeyRetryDelay: "--retry-delay",
 	}
 	lookupAndAddString(build, BuildGroup, stringKeys, podman)
 
@@ -1421,17 +1515,21 @@ func ConvertBuild(build *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isU
 	}
 	lookupAndAddAllStrings(build, BuildGroup, allStringKeys, podman)
 
-	annotations := build.LookupAllKeyVal(BuildGroup, KeyAnnotation)
+	annotations, warn := build.LookupAllKeyVal(BuildGroup, KeyAnnotation)
+	warnings = errors.Join(warnings, warn)
+
 	podman.addAnnotations(annotations)
 
-	podmanEnv := build.LookupAllKeyVal(BuildGroup, KeyEnvironment)
+	podmanEnv, warn := build.LookupAllKeyVal(BuildGroup, KeyEnvironment)
+	warnings = errors.Join(warnings, warn)
 	podman.addEnv(podmanEnv)
 
-	labels := build.LookupAllKeyVal(BuildGroup, KeyLabel)
+	labels, warn := build.LookupAllKeyVal(BuildGroup, KeyLabel)
+	warnings = errors.Join(warnings, warn)
 	podman.addLabels(labels)
 
 	if err := addNetworks(build, BuildGroup, service, unitsInfoMap, podman); err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	secrets := build.LookupAllArgs(BuildGroup, KeySecret)
@@ -1440,7 +1538,7 @@ func ConvertBuild(build *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isU
 	}
 
 	if err := addVolumes(build, service, BuildGroup, unitsInfoMap, podman); err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	// In order to build an image locally, we need either a File key pointing directly at a
@@ -1449,13 +1547,13 @@ func ConvertBuild(build *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isU
 	// an archive.
 	context, err := handleSetWorkingDirectory(build, service, BuildGroup)
 	if err != nil {
-		return nil, err
+		return nil, warnings, err
 	}
 
 	workingDirectory, okWD := service.Lookup(ServiceGroup, ServiceKeyWorkingDirectory)
 	filePath, okFile := build.Lookup(BuildGroup, KeyFile)
 	if (!okWD || len(workingDirectory) == 0) && (!okFile || len(filePath) == 0) && len(context) == 0 {
-		return nil, fmt.Errorf("neither SetWorkingDirectory, nor File key specified")
+		return nil, warnings, fmt.Errorf("neither SetWorkingDirectory, nor File key specified")
 	}
 
 	if len(filePath) > 0 {
@@ -1470,7 +1568,7 @@ func ConvertBuild(build *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isU
 	} else if !filepath.IsAbs(filePath) && !isURL(filePath) {
 		// Special handling for relative filePaths
 		if len(workingDirectory) == 0 {
-			return nil, fmt.Errorf("relative path in File key requires SetWorkingDirectory key to be set")
+			return nil, warnings, fmt.Errorf("relative path in File key requires SetWorkingDirectory key to be set")
 		}
 		podman.add(workingDirectory)
 	}
@@ -1478,7 +1576,7 @@ func ConvertBuild(build *parser.UnitFile, unitsInfoMap map[string]*UnitInfo, isU
 	service.AddCmdline(ServiceGroup, "ExecStart", podman.Args)
 
 	defaultOneshotServiceGroup(service, false)
-	return service, nil
+	return service, warnings, nil
 }
 
 func GetBuiltImageName(buildUnit *parser.UnitFile) string {
@@ -1533,6 +1631,10 @@ func ConvertPod(podUnit *parser.UnitFile, name string, unitsInfoMap map[string]*
 	service := podUnit.Dup()
 	service.Filename = unitInfo.ServiceFileName()
 
+	if err := translateUnitDependencies(service, unitsInfoMap); err != nil {
+		return nil, err
+	}
+
 	addDefaultDependencies(service, isUser)
 
 	if podUnit.Path != "" {
@@ -1609,10 +1711,11 @@ func ConvertPod(podUnit *parser.UnitFile, name string, unitsInfoMap map[string]*
 	}
 
 	stringsKeys := map[string]string{
-		KeyIP:  "--ip",
-		KeyIP6: "--ip6",
+		KeyIP:      "--ip",
+		KeyIP6:     "--ip6",
+		KeyShmSize: "--shm-size",
 	}
-	lookupAndAddAllStrings(podUnit, PodGroup, stringsKeys, execStartPre)
+	lookupAndAddString(podUnit, PodGroup, stringsKeys, execStartPre)
 
 	allStringsKeys := map[string]string{
 		KeyNetworkAlias: "--network-alias",
@@ -1620,6 +1723,7 @@ func ConvertPod(podUnit *parser.UnitFile, name string, unitsInfoMap map[string]*
 		KeyDNSOption:    "--dns-option",
 		KeyDNSSearch:    "--dns-search",
 		KeyAddHost:      "--add-host",
+		KeyHostName:     "--hostname",
 	}
 	lookupAndAddAllStrings(podUnit, PodGroup, allStringsKeys, execStartPre)
 
@@ -2198,3 +2302,62 @@ func addDefaultDependencies(service *parser.UnitFile, isUser bool) {
 		service.PrependUnitLine(UnitGroup, "Wants", networkUnit)
 	}
 }
+
+func handleExecReload(quadletUnitFile, serviceUnitFile *parser.UnitFile, groupName string) error {
+	reloadSignal, signalOk := quadletUnitFile.Lookup(groupName, KeyReloadSignal)
+	signalOk = signalOk && len(reloadSignal) > 0
+	reloadcmd, cmdOk := quadletUnitFile.LookupLastArgs(groupName, KeyReloadCmd)
+	cmdOk = cmdOk && len(reloadcmd) > 0
+
+	if !cmdOk && !signalOk {
+		return nil
+	}
+
+	if cmdOk && signalOk {
+		return fmt.Errorf("%s and %s are mutually exclusive but both are set", KeyReloadCmd, KeyReloadSignal)
+	}
+
+	serviceReloadCmd := createBasePodmanCommand(quadletUnitFile, groupName)
+	if cmdOk {
+		serviceReloadCmd.add("exec", "--cidfile=%t/%N.cid")
+		serviceReloadCmd.add(reloadcmd...)
+	} else {
+		serviceReloadCmd.add("kill", "--cidfile=%t/%N.cid", "--signal", reloadSignal)
+	}
+	serviceUnitFile.AddCmdline(ServiceGroup, "ExecReload", serviceReloadCmd.Args)
+
+	return nil
+}
+
+func translateUnitDependencies(serviceUnitFile *parser.UnitFile, unitsInfoMap map[string]*UnitInfo) error {
+	for _, unitDependencyKey := range unitDependencyKeys {
+		deps := serviceUnitFile.LookupAllStrv(UnitGroup, unitDependencyKey)
+		if len(deps) == 0 {
+			continue
+		}
+		translatedDeps := make([]string, 0, len(deps))
+		translated := false
+		for _, dep := range deps {
+			var translatedDep string
+
+			ext := filepath.Ext(dep)
+			if _, ok := SupportedExtensions[ext]; ok {
+				unitInfo, ok := unitsInfoMap[dep]
+				if !ok {
+					return fmt.Errorf("unable to translate dependency for %s", dep)
+				}
+				translatedDep = unitInfo.ServiceFileName()
+				translated = true
+			} else {
+				translatedDep = dep
+			}
+			translatedDeps = append(translatedDeps, translatedDep)
+		}
+		if !translated {
+			continue
+		}
+		serviceUnitFile.Unset(UnitGroup, unitDependencyKey)
+		serviceUnitFile.Add(UnitGroup, unitDependencyKey, strings.Join(translatedDeps, " "))
+	}
+	return nil
+}
